@@ -431,6 +431,7 @@ update_account_object (GoaDaemon           *daemon,
 {
   GoaAccount *account;
   GoaProvider *provider;
+  gboolean is_locked;
   gboolean ret;
   gchar *identity;
   gchar *presentation_identity;
@@ -458,6 +459,7 @@ update_account_object (GoaDaemon           *daemon,
   type = g_key_file_get_string (key_file, group, "Provider", NULL);
   identity = g_key_file_get_string (key_file, group, "Identity", NULL);
   presentation_identity = g_key_file_get_string (key_file, group, "PresentationIdentity", NULL);
+  is_locked = g_key_file_get_boolean (key_file, group, "IsLocked", NULL);
   if (just_added)
     {
       account = goa_account_skeleton_new ();
@@ -479,6 +481,7 @@ update_account_object (GoaDaemon           *daemon,
   goa_account_set_provider_type (account, type);
   goa_account_set_identity (account, identity);
   goa_account_set_presentation_identity (account, presentation_identity);
+  goa_account_set_is_locked (account, is_locked);
 
   error = NULL;
   if (!goa_provider_build_object (provider, object, key_file, group, daemon->connection, just_added, &error))
@@ -508,7 +511,6 @@ update_account_object (GoaDaemon           *daemon,
   g_object_unref (account);
   g_free (type);
   g_free (identity);
-  g_free (presentation_identity);
   return ret;
 }
 
@@ -585,7 +587,6 @@ process_config_entries (GoaDaemon  *daemon,
       g_signal_handlers_disconnect_by_func (goa_object_peek_account (object),
                                             G_CALLBACK (on_account_handle_remove),
                                             daemon);
-      g_object_unref (object);
       g_debug ("removing %s", object_path);
       g_warn_if_fail (g_dbus_object_manager_server_unexport (daemon->object_manager, object_path));
     }
@@ -821,7 +822,9 @@ get_all_providers_cb (GObject      *source,
         }
     }
 
-  id = generate_new_id (data->daemon);
+  if (!g_variant_lookup (data->details, "Id", "s", &id))
+    id = generate_new_id (data->daemon);
+
   group = g_strdup_printf ("Account %s", id);
   g_key_file_set_string (key_file, group, "Provider", data->provider_type);
   g_key_file_set_string (key_file, group, "Identity", data->identity);
@@ -845,6 +848,10 @@ get_all_providers_cb (GObject      *source,
             }
         }
 
+      /* Skip Id since we already handled it above. */
+      if (g_strcmp0 (key, "Id") == 0)
+        continue;
+
       g_key_file_set_string (key_file, group, key, value);
     }
 
@@ -959,6 +966,17 @@ on_account_handle_remove (GoaAccount            *account,
   key_file = NULL;
   data = NULL;
 
+  if (goa_account_get_is_locked (account))
+    {
+      error = NULL;
+      g_set_error_literal (&error,
+                           GOA_ERROR,
+                           GOA_ERROR_NOT_SUPPORTED,
+                           _("IsLocked property is set for account"));
+      g_dbus_method_invocation_take_error (invocation, error);
+      goto out;
+    }
+
   /* update key-file - right now we only support removing the account
    * if the entry is in ~/.config/goa-1.0/accounts.conf
    */
@@ -1159,7 +1177,7 @@ on_account_handle_ensure_credentials (GoaAccount            *account,
                                       gpointer               user_data)
 {
   GoaDaemon *daemon = GOA_DAEMON (user_data);
-  GoaProvider *provider = NULL;
+  GoaProvider *provider;
   GoaObject *object;
 
   object = GOA_OBJECT (g_dbus_interface_get_object (G_DBUS_INTERFACE (account)));
@@ -1182,6 +1200,5 @@ on_account_handle_ensure_credentials (GoaAccount            *account,
                                    ensure_data_new (daemon, object, invocation));
 
  out:
-  g_clear_object (&provider);
   return TRUE; /* invocation was handled */
 }
